import enum
import queue
import datetime
import collections
import logging


LOG = logging.getLogger(__name__)


class Event(dict):
    Type = enum.Enum("Type", "UPLOADED,INSTALLED,FAILED,MIGRATED,REMOVED")

    def __init__(self, typ, package, version, distribution, extra=None, user=None, error=None):
        super().__init__()
        self["timestamp"] = str(datetime.datetime.now())
        self["typ"] = typ.name
        self["package"] = package
        self["version"] = version
        self["distribution"] = distribution
        self["extra"] = extra
        self["user"] = user
        self["error"] = error


class Log(collections.deque):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._attached = {}

    def event(self, *args, **kwargs):
        event = Event(*args, **kwargs)
        self.appendleft(event)
        for k, q in self._attached.items():
            LOG.info(f"Multiplexing event to attached queue: {k}")
            q.put(event)

    def attach(self, key):
        return self._attached.setdefault(key, queue.Queue())

    def detach(self, key):
        del self._attached[key]

    def get_last_packages(self):
        return sorted({e["package"] for e in self})

    def get_last_versions(self, package):
        return sorted({e["version"] for e in self if e["package"] == package})
